@@ -21,6 +21,12 @@ struct ColumnMajor : public BaseTensorLayout
 {
     static constexpr const char* name = "ColumnMajor";
 };
+
+struct MFMA : public BaseTensorLayout
+{
+    static constexpr const char* name = "MFMA";
+};
+
 } // namespace gemm
 
 namespace convolution {
@@ -430,7 +436,7 @@ struct G_NDHW : public BaseTensorLayout
 
 } // namespace convolution
 
-#ifndef CK_CODE_GEN_RTC
+#if !defined(__HIPCC_RTC__) || !defined(CK_CODE_GEN_RTC)
 template <
     typename Layout,
     typename std::enable_if<std::is_base_of<BaseTensorLayout, Layout>::value, bool>::type = false>
