@@ -1,10 +1,9 @@
 
 // SPDX-License-Identifier: MIT
-// Copyright (c) 2018-2024, Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2018-2025, Advanced Micro Devices, Inc. All rights reserved.
 
 #pragma once
 
-#include "ck/library/utility/numeric.hpp"
 #include "ck/utility/common_header.hpp"
 #include "ck/tensor_description/tensor_descriptor.hpp"
 #include "ck/tensor_description/tensor_descriptor_helper.hpp"
@@ -20,7 +19,8 @@ template <index_t NDimSpatial,
           typename ADataType       = float,
           typename CDataType       = float,
           index_t NumGroupsToMerge = 1,
-          typename IndexType       = index_t>
+          typename IndexType       = index_t,
+          bool CTranspose          = false>
 struct TransformConvFwdToGemm
 {
     private:
@@ -84,9 +84,10 @@ struct TransformConvFwdToGemm
             }
             else
             {
-                // Not possible to support even after split N.
-                // Too large tensor.
-                return N;
+                // Split Convolution's N dimension into N workgroups. However
+                // this still might not result in sufficiently small tensor,
+                // but at least later on we could divide the image as well.
+                return 1;
             }
         }
         else
@@ -148,8 +149,8 @@ struct TransformConvFwdToGemm
 
     template <typename ConvDimsType,
               typename ConvSpatialDimsType,
-              index_t NDim                                   = NDimSpatial,
-              typename std::enable_if<NDim == 1, bool>::type = false>
+              index_t NDim                                  = NDimSpatial,
+              typename ck::enable_if<NDim == 1, bool>::type = false>
     __host__ __device__ TransformConvFwdToGemm(const ConvDimsType& a_g_n_c_wis_lengths,
                                                const ConvDimsType& a_g_n_c_wis_strides,
                                                const ConvDimsType& b_g_k_c_xs_lengths,
@@ -201,11 +202,15 @@ struct TransformConvFwdToGemm
           InRightPadW_{input_right_pads[I0]},
           ZYX_{X_}
     {
+#ifdef CK_CODE_GEN_RTC
+        static_assert(is_same_v<ConvSpatialDimsType, ck::Array<IndexType, NDimSpatial>>);
+        static_assert(is_same_v<ConvDimsType, ck::Array<IndexType, NDimSpatial + I3>>);
+#else
         static_assert(is_same_v<ConvSpatialDimsType, std::array<IndexType, NDimSpatial>> ||
                       is_same_v<ConvSpatialDimsType, ck::Array<IndexType, NDimSpatial>>);
         static_assert(is_same_v<ConvDimsType, std::array<IndexType, NDimSpatial + I3>> ||
                       is_same_v<ConvDimsType, ck::Array<IndexType, NDimSpatial + I3>>);
-
+#endif
         if constexpr(SplitN)
         {
             N_ = GetSplitedNSize(
@@ -219,8 +224,8 @@ struct TransformConvFwdToGemm
 
     template <typename ConvDimsType,
               typename ConvSpatialDimsType,
-              index_t NDim                                   = NDimSpatial,
-              typename std::enable_if<NDim == 2, bool>::type = false>
+              index_t NDim                                  = NDimSpatial,
+              typename ck::enable_if<NDim == 2, bool>::type = false>
     __host__ __device__ TransformConvFwdToGemm(const ConvDimsType& a_g_n_c_wis_lengths,
                                                const ConvDimsType& a_g_n_c_wis_strides,
                                                const ConvDimsType& b_g_k_c_xs_lengths,
@@ -272,11 +277,15 @@ struct TransformConvFwdToGemm
           InRightPadW_{input_right_pads[I1]},
           ZYX_{Y_ * X_}
     {
+#ifdef CK_CODE_GEN_RTC
+        static_assert(is_same_v<ConvSpatialDimsType, ck::Array<IndexType, NDimSpatial>>);
+        static_assert(is_same_v<ConvDimsType, ck::Array<IndexType, NDimSpatial + I3>>);
+#else
         static_assert(is_same_v<ConvSpatialDimsType, std::array<IndexType, NDimSpatial>> ||
                       is_same_v<ConvSpatialDimsType, ck::Array<IndexType, NDimSpatial>>);
         static_assert(is_same_v<ConvDimsType, std::array<IndexType, NDimSpatial + I3>> ||
                       is_same_v<ConvDimsType, ck::Array<IndexType, NDimSpatial + I3>>);
-
+#endif
         if constexpr(SplitN)
         {
             N_ = GetSplitedNSize(
@@ -290,8 +299,8 @@ struct TransformConvFwdToGemm
 
     template <typename ConvDimsType,
               typename ConvSpatialDimsType,
-              index_t NDim                                   = NDimSpatial,
-              typename std::enable_if<NDim == 3, bool>::type = false>
+              index_t NDim                                  = NDimSpatial,
+              typename ck::enable_if<NDim == 3, bool>::type = false>
     __host__ __device__ TransformConvFwdToGemm(const ConvDimsType& a_g_n_c_wis_lengths,
                                                const ConvDimsType& a_g_n_c_wis_strides,
                                                const ConvDimsType& b_g_k_c_xs_lengths,
@@ -343,11 +352,15 @@ struct TransformConvFwdToGemm
           InRightPadW_{input_right_pads[I2]},
           ZYX_{Z_ * Y_ * X_}
     {
+#ifdef CK_CODE_GEN_RTC
+        static_assert(is_same_v<ConvSpatialDimsType, ck::Array<IndexType, NDimSpatial>>);
+        static_assert(is_same_v<ConvDimsType, ck::Array<IndexType, NDimSpatial + I3>>);
+#else
         static_assert(is_same_v<ConvSpatialDimsType, std::array<IndexType, NDimSpatial>> ||
                       is_same_v<ConvSpatialDimsType, ck::Array<IndexType, NDimSpatial>>);
         static_assert(is_same_v<ConvDimsType, std::array<IndexType, NDimSpatial + I3>> ||
                       is_same_v<ConvDimsType, ck::Array<IndexType, NDimSpatial + I3>>);
-
+#endif
         if constexpr(SplitN)
         {
             N_ = GetSplitedNSize(
@@ -478,11 +491,11 @@ struct TransformConvFwdToGemm
     // TODO: implement ck::tensor_layout::convolution that describe packed/strided dimemsion as
     // properties
     template <typename ALayout,
-              typename std::enable_if<NDimSpatial == 1 &&
-                                          (is_same_v<ALayout, tensor_layout::convolution::G_NW_C> ||
-                                           is_same_v<ALayout, tensor_layout::convolution::NWGC> ||
-                                           is_same_v<ALayout, tensor_layout::convolution::GNWC>),
-                                      bool>::type = false>
+              typename ck::enable_if<NDimSpatial == 1 &&
+                                         (is_same_v<ALayout, tensor_layout::convolution::G_NW_C> ||
+                                          is_same_v<ALayout, tensor_layout::convolution::NWGC> ||
+                                          is_same_v<ALayout, tensor_layout::convolution::GNWC>),
+                                     bool>::type = false>
     __host__ __device__ auto MakeADescriptor_M_K() const
     {
         if constexpr(ConvForwardSpecialization ==
@@ -691,11 +704,11 @@ struct TransformConvFwdToGemm
     }
 
     template <typename ALayout,
-              typename std::enable_if<
-                  NDimSpatial == 2 && (is_same_v<ALayout, tensor_layout::convolution::G_NHW_C> ||
-                                       is_same_v<ALayout, tensor_layout::convolution::NHWGC> ||
-                                       is_same_v<ALayout, tensor_layout::convolution::GNHWC>),
-                  bool>::type = false>
+              typename ck::enable_if<NDimSpatial == 2 &&
+                                         (is_same_v<ALayout, tensor_layout::convolution::G_NHW_C> ||
+                                          is_same_v<ALayout, tensor_layout::convolution::NHWGC> ||
+                                          is_same_v<ALayout, tensor_layout::convolution::GNHWC>),
+                                     bool>::type = false>
     __host__ __device__ auto MakeADescriptor_M_K() const
 
     {
@@ -932,7 +945,7 @@ struct TransformConvFwdToGemm
     }
 
     template <typename ALayout,
-              typename std::enable_if<
+              typename ck::enable_if<
                   NDimSpatial == 3 && (is_same_v<ALayout, tensor_layout::convolution::G_NDHW_C> ||
                                        is_same_v<ALayout, tensor_layout::convolution::NDHWGC> ||
                                        is_same_v<ALayout, tensor_layout::convolution::GNDHWC>),
@@ -1241,20 +1254,97 @@ struct TransformConvFwdToGemm
         }
     }
 
+    template <typename ALayout,
+              typename ck::enable_if<NDimSpatial == 1 &&
+                                         is_same_v<ALayout, tensor_layout::convolution::NGCW>,
+                                     bool>::type = false>
+    __host__ __device__ auto MakeADescriptor_M_K() const
+    {
+        static_assert(NumGroupsToMerge == 1);
+        static_assert(ConvForwardSpecialization ==
+                      device::ConvolutionForwardSpecialization::Filter1x1Stride1Pad0);
+
+        const auto in_gemmm_gemmk_desc = make_naive_tensor_descriptor(
+            make_tuple(N_, Wo_, C_), make_tuple(NStrideTensorA_, I1, CStrideTensorA_));
+
+        return transform_tensor_descriptor(
+            in_gemmm_gemmk_desc,
+            make_tuple(make_merge_transform(make_tuple(N_, Wo_)), make_pass_through_transform(C_)),
+            make_tuple(Sequence<0, 1>{}, Sequence<2>{}),
+            make_tuple(Sequence<0>{}, Sequence<1>{}));
+    }
+
+    template <typename ALayout,
+              typename ck::enable_if<NDimSpatial == 2 &&
+                                         is_same_v<ALayout, tensor_layout::convolution::NGCHW>,
+                                     bool>::type = false>
+    __host__ __device__ auto MakeADescriptor_M_K() const
+    {
+        static_assert(NumGroupsToMerge == 1);
+        static_assert(ConvForwardSpecialization ==
+                      device::ConvolutionForwardSpecialization::Filter1x1Stride1Pad0);
+
+        const auto in_gemmm_gemmk_desc = make_naive_tensor_descriptor(
+            make_tuple(N_, Ho_ * Wo_, C_), make_tuple(NStrideTensorA_, I1, CStrideTensorA_));
+
+        return transform_tensor_descriptor(
+            in_gemmm_gemmk_desc,
+            make_tuple(make_merge_transform(make_tuple(N_, Ho_ * Wo_)),
+                       make_pass_through_transform(C_)),
+            make_tuple(Sequence<0, 1>{}, Sequence<2>{}),
+            make_tuple(Sequence<0>{}, Sequence<1>{}));
+    }
+
+    template <typename ALayout,
+              typename ck::enable_if<NDimSpatial == 3 &&
+                                         is_same_v<ALayout, tensor_layout::convolution::NGCDHW>,
+                                     bool>::type = false>
+    __host__ __device__ auto MakeADescriptor_M_K() const
+    {
+        static_assert(NumGroupsToMerge == 1);
+        static_assert(ConvForwardSpecialization ==
+                      device::ConvolutionForwardSpecialization::Filter1x1Stride1Pad0);
+
+        const auto in_gemmm_gemmk_desc = make_naive_tensor_descriptor(
+            make_tuple(N_, Do_ * Ho_ * Wo_, C_), make_tuple(NStrideTensorA_, I1, CStrideTensorA_));
+
+        return transform_tensor_descriptor(
+            in_gemmm_gemmk_desc,
+            make_tuple(make_merge_transform(make_tuple(N_, Do_ * Ho_ * Wo_)),
+                       make_pass_through_transform(C_)),
+            make_tuple(Sequence<0, 1>{}, Sequence<2>{}),
+            make_tuple(Sequence<0>{}, Sequence<1>{}));
+    }
+
     template <typename BLayout,
-              typename std::enable_if<is_same_v<BLayout, tensor_layout::convolution::GKXC> ||
-                                          is_same_v<BLayout, tensor_layout::convolution::GKYXC> ||
-                                          is_same_v<BLayout, tensor_layout::convolution::GKZYXC>,
-                                      bool>::type = false>
+              typename ck::enable_if<is_same_v<BLayout, tensor_layout::convolution::GKCX> ||
+                                         is_same_v<BLayout, tensor_layout::convolution::GKCYX> ||
+                                         is_same_v<BLayout, tensor_layout::convolution::GKCZYX>,
+                                     bool>::type = false>
+    __host__ __device__ auto MakeBDescriptor_N_K() const
+    {
+        static_assert(ConvForwardSpecialization ==
+                          device::ConvolutionForwardSpecialization::Filter1x1Stride1Pad0 ||
+                      ConvForwardSpecialization ==
+                          device::ConvolutionForwardSpecialization::Filter1x1Pad0);
+        static_assert(NumGroupsToMerge == 1);
+        return make_naive_tensor_descriptor_packed(make_tuple(K_, C_));
+    }
+
+    template <typename BLayout,
+              typename ck::enable_if<is_same_v<BLayout, tensor_layout::convolution::GKXC> ||
+                                         is_same_v<BLayout, tensor_layout::convolution::GKYXC> ||
+                                         is_same_v<BLayout, tensor_layout::convolution::GKZYXC>,
+                                     bool>::type = false>
     __host__ __device__ auto MakeBDescriptor_N_K() const
     {
         if constexpr(ConvForwardSpecialization ==
                      device::ConvolutionForwardSpecialization::Filter3x3)
         {
             using FilterSizeNumType =
-                std::conditional_t<NDimSpatial == 1,
-                                   Number<3>,
-                                   std::conditional_t<NDimSpatial == 2, Number<9>, Number<27>>>;
+                ck::conditional_t<NDimSpatial == 1,
+                                  Number<3>,
+                                  ck::conditional_t<NDimSpatial == 2, Number<9>, Number<27>>>;
 
             if constexpr(NumGroupsToMerge == 1)
             {
@@ -1297,13 +1387,13 @@ struct TransformConvFwdToGemm
 
     template <
         typename BLayout,
-        typename std::enable_if<is_same_v<BLayout, tensor_layout::convolution::G_K_X_C> ||
-                                    is_same_v<BLayout, tensor_layout::convolution::G_K_YX_C> ||
-                                    is_same_v<BLayout, tensor_layout::convolution::G_K_ZYX_C> ||
-                                    is_same_v<BLayout, tensor_layout::convolution::KXGC> ||
-                                    is_same_v<BLayout, tensor_layout::convolution::KYXGC> ||
-                                    is_same_v<BLayout, tensor_layout::convolution::KZYXGC>,
-                                bool>::type = false>
+        typename ck::enable_if<is_same_v<BLayout, tensor_layout::convolution::G_K_X_C> ||
+                                   is_same_v<BLayout, tensor_layout::convolution::G_K_YX_C> ||
+                                   is_same_v<BLayout, tensor_layout::convolution::G_K_ZYX_C> ||
+                                   is_same_v<BLayout, tensor_layout::convolution::KXGC> ||
+                                   is_same_v<BLayout, tensor_layout::convolution::KYXGC> ||
+                                   is_same_v<BLayout, tensor_layout::convolution::KZYXGC>,
+                               bool>::type = false>
     __host__ __device__ auto MakeBDescriptor_N_K() const
     {
         const auto wei_k_yx_c_desc = make_naive_tensor_descriptor(
@@ -1318,51 +1408,77 @@ struct TransformConvFwdToGemm
         return wei_gemmn_gemmk_desc;
     }
 
-    template <typename CLayout,
-              index_t NDimSp = NDimSpatial,
+    template <
+        typename CLayout,
+        index_t NDimSp = NDimSpatial,
 
-              typename std::enable_if<NDimSp == 1 &&
-                                          (is_same_v<CLayout, tensor_layout::convolution::G_K>),
-                                      bool>::type = false>
+        typename ck::enable_if<NDimSp == 1 && (is_same_v<CLayout, tensor_layout::convolution::G_K>),
+                               bool>::type = false>
     __host__ __device__ auto MakeCDescriptor_M_N() const
     {
-        return make_naive_tensor_descriptor(make_tuple(N_ * Wo_, K_),
-                                            make_tuple(I0, KStrideTensorC_));
+        if constexpr(CTranspose)
+        {
+            return make_naive_tensor_descriptor(make_tuple(K_, N_ * Wo_),
+                                                make_tuple(KStrideTensorC_, I0));
+        }
+        else
+        {
+            return make_naive_tensor_descriptor(make_tuple(N_ * Wo_, K_),
+                                                make_tuple(I0, KStrideTensorC_));
+        }
+    }
+
+    template <
+        typename CLayout,
+        index_t NDimSp = NDimSpatial,
+
+        typename ck::enable_if<NDimSp == 2 && (is_same_v<CLayout, tensor_layout::convolution::G_K>),
+                               bool>::type = false>
+    __host__ __device__ auto MakeCDescriptor_M_N() const
+    {
+        if constexpr(CTranspose)
+        {
+            return make_naive_tensor_descriptor(make_tuple(K_, N_ * Ho_ * Wo_),
+                                                make_tuple(KStrideTensorC_, I0));
+        }
+        else
+        {
+            return make_naive_tensor_descriptor(make_tuple(N_ * Ho_ * Wo_, K_),
+                                                make_tuple(I0, KStrideTensorC_));
+        }
+    }
+
+    template <
+        typename CLayout,
+        index_t NDimSp = NDimSpatial,
+
+        typename ck::enable_if<NDimSp == 3 && (is_same_v<CLayout, tensor_layout::convolution::G_K>),
+                               bool>::type = false>
+    __host__ __device__ auto MakeCDescriptor_M_N() const
+    {
+        if constexpr(CTranspose)
+        {
+            return make_naive_tensor_descriptor(make_tuple(K_, N_ * Do_ * Ho_ * Wo_),
+                                                make_tuple(KStrideTensorC_, I0));
+        }
+        else
+        {
+            return make_naive_tensor_descriptor(make_tuple(N_ * Do_ * Ho_ * Wo_, K_),
+                                                make_tuple(I0, KStrideTensorC_));
+        }
     }
 
     template <typename CLayout,
               index_t NDimSp = NDimSpatial,
 
-              typename std::enable_if<NDimSp == 2 &&
-                                          (is_same_v<CLayout, tensor_layout::convolution::G_K>),
-                                      bool>::type = false>
-    __host__ __device__ auto MakeCDescriptor_M_N() const
-    {
-        return make_naive_tensor_descriptor(make_tuple(N_ * Ho_ * Wo_, K_),
-                                            make_tuple(I0, KStrideTensorC_));
-    }
-
-    template <typename CLayout,
-              index_t NDimSp = NDimSpatial,
-
-              typename std::enable_if<NDimSp == 3 &&
-                                          (is_same_v<CLayout, tensor_layout::convolution::G_K>),
-                                      bool>::type = false>
-    __host__ __device__ auto MakeCDescriptor_M_N() const
-    {
-        return make_naive_tensor_descriptor(make_tuple(N_ * Do_ * Ho_ * Wo_, K_),
-                                            make_tuple(I0, KStrideTensorC_));
-    }
-
-    template <typename CLayout,
-              index_t NDimSp                      = NDimSpatial,
-              typename std::enable_if<NDimSp == 1 &&
-                                          (is_same_v<CLayout, tensor_layout::convolution::G_NW_K> ||
-                                           is_same_v<CLayout, tensor_layout::convolution::NWGK> ||
-                                           is_same_v<CLayout, tensor_layout::convolution::GNWK>),
-                                      bool>::type = false>
+              typename ck::enable_if<NDimSp == 1 &&
+                                         (is_same_v<CLayout, tensor_layout::convolution::G_NW_K> ||
+                                          is_same_v<CLayout, tensor_layout::convolution::NWGK> ||
+                                          is_same_v<CLayout, tensor_layout::convolution::GNWK>),
+                                     bool>::type = false>
     __host__ __device__ auto MakeCDescriptor_M_N() const
     {
+        static_assert(CTranspose == false);
         const IndexType NDoHoWo = N_ * Wo_;
         if constexpr(NumGroupsToMerge == 1)
         {
@@ -1410,13 +1526,14 @@ struct TransformConvFwdToGemm
     template <typename CLayout,
               index_t NDimSp = NDimSpatial,
 
-              typename std::enable_if<
-                  NDimSp == 2 && (is_same_v<CLayout, tensor_layout::convolution::G_NHW_K> ||
-                                  is_same_v<CLayout, tensor_layout::convolution::NHWGK> ||
-                                  is_same_v<CLayout, tensor_layout::convolution::GNHWK>),
-                  bool>::type = false>
+              typename ck::enable_if<NDimSp == 2 &&
+                                         (is_same_v<CLayout, tensor_layout::convolution::G_NHW_K> ||
+                                          is_same_v<CLayout, tensor_layout::convolution::NHWGK> ||
+                                          is_same_v<CLayout, tensor_layout::convolution::GNHWK>),
+                                     bool>::type = false>
     __host__ __device__ auto MakeCDescriptor_M_N() const
     {
+        static_assert(CTranspose == false);
         const IndexType NDoHoWo = N_ * Ho_ * Wo_;
         if constexpr(NumGroupsToMerge == 1)
         {
@@ -1467,14 +1584,14 @@ struct TransformConvFwdToGemm
 
     template <typename CLayout,
               index_t NDimSp = NDimSpatial,
-              typename std::enable_if<
+              typename ck::enable_if<
                   NDimSp == 3 && (is_same_v<CLayout, tensor_layout::convolution::G_NDHW_K> ||
                                   is_same_v<CLayout, tensor_layout::convolution::NDHWGK> ||
                                   is_same_v<CLayout, tensor_layout::convolution::GNDHWK>),
                   bool>::type = false>
     __host__ __device__ auto MakeCDescriptor_M_N() const
     {
-
+        static_assert(CTranspose == false);
         const IndexType NDoHoWo = N_ * Do_ * Ho_ * Wo_;
         if constexpr(NumGroupsToMerge == 1)
         {
@@ -1524,6 +1641,101 @@ struct TransformConvFwdToGemm
         }
     }
 
+    template <typename CLayout,
+              index_t NDimSp = NDimSpatial,
+
+              typename ck::enable_if<NDimSp == 1 &&
+                                         (is_same_v<CLayout, tensor_layout::convolution::GNKW> ||
+                                          is_same_v<CLayout, tensor_layout::convolution::NGKW>),
+                                     bool>::type = false>
+    __host__ __device__ auto MakeCDescriptor_M_N() const
+    {
+        static_assert(NumGroupsToMerge == 1);
+        auto n_k_wo_desc = make_naive_tensor_descriptor(
+            make_tuple(N_, K_, Wo_), make_tuple(NStrideTensorC_, KStrideTensorC_, I1));
+        if constexpr(CTranspose)
+        {
+            return transform_tensor_descriptor(
+                n_k_wo_desc,
+                make_tuple(make_pass_through_transform(K_),
+                           make_merge_transform(make_tuple(N_, Wo_))),
+                make_tuple(Sequence<1>{}, Sequence<0, 2>{}),
+                make_tuple(Sequence<0>{}, Sequence<1>{}));
+        }
+        else
+        {
+            return transform_tensor_descriptor(n_k_wo_desc,
+                                               make_tuple(make_merge_transform(make_tuple(N_, Wo_)),
+                                                          make_pass_through_transform(K_)),
+                                               make_tuple(Sequence<0, 2>{}, Sequence<1>{}),
+                                               make_tuple(Sequence<0>{}, Sequence<1>{}));
+        }
+    }
+
+    template <typename CLayout,
+              index_t NDimSp = NDimSpatial,
+
+              typename ck::enable_if<NDimSp == 2 &&
+                                         (is_same_v<CLayout, tensor_layout::convolution::GNKHW> ||
+                                          is_same_v<CLayout, tensor_layout::convolution::NGKHW>),
+                                     bool>::type = false>
+    __host__ __device__ auto MakeCDescriptor_M_N() const
+    {
+        static_assert(NumGroupsToMerge == 1);
+        auto n_k_howo_desc = make_naive_tensor_descriptor(
+            make_tuple(N_, K_, Ho_ * Wo_), make_tuple(NStrideTensorC_, KStrideTensorC_, I1));
+        if constexpr(CTranspose)
+        {
+            return transform_tensor_descriptor(
+                n_k_howo_desc,
+                make_tuple(make_pass_through_transform(K_),
+                           make_merge_transform(make_tuple(N_, Ho_ * Wo_))),
+                make_tuple(Sequence<1>{}, Sequence<0, 2>{}),
+                make_tuple(Sequence<0>{}, Sequence<1>{}));
+        }
+        else
+        {
+            return transform_tensor_descriptor(
+                n_k_howo_desc,
+                make_tuple(make_merge_transform(make_tuple(N_, Ho_ * Wo_)),
+                           make_pass_through_transform(K_)),
+                make_tuple(Sequence<0, 2>{}, Sequence<1>{}),
+                make_tuple(Sequence<0>{}, Sequence<1>{}));
+        }
+    }
+
+    template <typename CLayout,
+              index_t NDimSp = NDimSpatial,
+
+              typename ck::enable_if<NDimSp == 3 &&
+                                         (is_same_v<CLayout, tensor_layout::convolution::GNKDHW> ||
+                                          is_same_v<CLayout, tensor_layout::convolution::NGKDHW>),
+                                     bool>::type = false>
+    __host__ __device__ auto MakeCDescriptor_M_N() const
+    {
+        static_assert(NumGroupsToMerge == 1);
+        auto n_k_dohowo_desc = make_naive_tensor_descriptor(
+            make_tuple(N_, K_, Do_ * Ho_ * Wo_), make_tuple(NStrideTensorC_, KStrideTensorC_, I1));
+
+        if constexpr(CTranspose)
+        {
+            return transform_tensor_descriptor(
+                n_k_dohowo_desc,
+                make_tuple(make_pass_through_transform(K_),
+                           make_merge_transform(make_tuple(N_, Do_ * Ho_ * Wo_))),
+                make_tuple(Sequence<1>{}, Sequence<0, 2>{}),
+                make_tuple(Sequence<0>{}, Sequence<1>{}));
+        }
+        else
+        {
+            return transform_tensor_descriptor(
+                n_k_dohowo_desc,
+                make_tuple(make_merge_transform(make_tuple(N_, Do_ * Ho_ * Wo_)),
+                           make_pass_through_transform(K_)),
+                make_tuple(Sequence<0, 2>{}, Sequence<1>{}),
+                make_tuple(Sequence<0>{}, Sequence<1>{}));
+        }
+    }
     IndexType N_;
     IndexType Di_, Hi_, Wi_;
     IndexType Do_, Ho_, Wo_;
